/* $Id: dir.c,v 1.2 2003/01/18 12:20:46 pho Exp $ */

/* Create deap directories using relative paths */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>

static int max;

void
error(char *op, char* arg, char* file, int line) {
	fprintf(stderr,"%s. %s. %s (%s:%d)\n",
		op, arg, sys_errlist[errno], file, line);
}

void
mkDir(char *path, int level) {
	char newPath[10];

	if (mkdir(path, 0770) == -1) {
		error("mkdir", path, __FILE__, __LINE__);
		fprintf(stderr, "length(path) = %d\n", strlen(path));
		fprintf(stderr, ") level = %d\n", level);
		exit(2);
	}
	chdir(path);
	if (level < max) {
		sprintf(newPath,"d%d", level+1);
		mkDir(newPath, level+1);
	}
}

void
rmDir(char *path, int level) {
	char newPath[10];

	if (level < max) {
		sprintf(newPath,"d%d", level+1);
		rmDir(newPath, level+1);
	}
	chdir ("..");
	if (rmdir(path) == -1) {
		error("rmdir", path, __FILE__, __LINE__);
		exit(2);
	}
}

void
rmDir2(char *path, int level) {
	char newPath[10];

	chdir(path);
	sprintf(newPath,"d%d", level+1);
	if (access(newPath, R_OK) == 0) 
		rmDir2(newPath, level+1);
	chdir ("..");
	if (rmdir(path) == -1) {
		error("rmdir", path, __FILE__, __LINE__);
		exit(2);
	}
}

int
main(int argc, char **argv) {
	int c, levels = 1, pid, leave = 0;
	char path[128], rpath[128] = "";
	extern char *optarg;

	while ((c = getopt(argc, argv, "ln:r:")) != -1)
		switch (c) {
			case 'l':
				leave = 1;
				break;
			case 'r':
				strcpy(rpath, optarg);
				break;
			case 'n':
				levels = atoi(optarg);
				break;
			default:
				fprintf(stderr,
					"Usage: %s {-l} | {-n <num>} | -r <dir>\n",
					argv[0]);
				exit(1);
		}


	max = levels;
	pid = getpid();

	if (strlen(rpath) > 0) {
		rmDir2(rpath,1);
	} else {
		umask(0);
		sprintf(path,"p%05d.d%d", pid, 1);
		mkDir(path, 1);
		if (leave == 0) rmDir(path, 1);
	}
	return 0;
}
